#!/usr/bin/python
# -*-coding:utf-8-*-

"""
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions: 
* 
* The above notice and this permission notice shall be included in all copies
* or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* File for "Mipmapping" lesson of the OpenGL tutorial on
* www.videotutorialsrock.com
*
* File in python created by Sehnoax based on files from 
* www.videotutorialsrock.com
"""

import sys

from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL.GLUT import *
from PIL import Image


class myGLCanvas(object):
    def __init__(self):
        self._angle = 30.
        self.pos = 0
        self.floorTextureSize = 15.

        glutInit(sys.argv)
        glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH)
        glutInitWindowSize(400, 400)
        
        glutCreateWindow(b"Mipmapping - videotutorialsrock.com")

        self.initRendering()
        self.loadTexture()
  
        glutDisplayFunc(self.OnPaint)
        glutReshapeFunc(self.OnResize)
        glutKeyboardFunc(self.handleKeypress)

        glutTimerFunc(25, self.update, 0)        

        glutMainLoop()
        return 0


    def initRendering(self):
        glEnable(GL_DEPTH_TEST)
        glEnable(GL_COLOR_MATERIAL)
        glEnable(GL_LIGHTING)
        glEnable(GL_LIGHT0)
        glEnable(GL_LIGHT1)
        glEnable(GL_NORMALIZE)
        
        
    def OnPaint(self):
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
        
        glMatrixMode(GL_MODELVIEW)
        glLoadIdentity()
        
        glTranslatef(0.0, 0.0, -20.0)
        glRotatef(-self._angle, 0.0, 1.0, 0.0)

        ambientLight = [0.3, 0.3, 0.3, 1.0]
        glLightModelfv(GL_LIGHT_MODEL_AMBIENT, ambientLight)
        
        directedLight = [0.7, 0.7, 0.7, 1.0]
        directedLightPos = [-0.2, 0.3, -1, 0.0]
        glLightfv(GL_LIGHT0, GL_DIFFUSE, directedLight)
        glLightfv(GL_LIGHT0, GL_POSITION, directedLightPos)

        glTranslatef(0.0, -5.4, 0.0)
        glEnable(GL_TEXTURE_2D)
        glBindTexture(GL_TEXTURE_2D, self.textureId)
        #glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)
        #glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
        glTexParameteri(GL_TEXTURE_2D, \
			GL_TEXTURE_MIN_FILTER, \
			GL_LINEAR_MIPMAP_LINEAR)
        
        glBegin(GL_QUADS)
        
        glNormal3f(0.0, 1.0, 0.0)
        glTexCoord2f(2000/self.floorTextureSize, \
                     self.pos/self.floorTextureSize)
        glVertex3f(-1000.0, 0.0, -1000.0)
        glTexCoord2f(2000 / self.floorTextureSize, \
                     (2000 + self.pos)/self.floorTextureSize)
        glVertex3f(-1000.0, 0.0, 1000.0)
        glTexCoord2f(0.0, (2000 + self.pos)/self.floorTextureSize)
        glVertex3f(1000.0, 0.0, 1000.0)
        glTexCoord2f(0.0, self.pos/self.floorTextureSize)
        glVertex3f(1000.0, 0.0, -1000.0)
        
        glEnd()
        
        glutSwapBuffers()
          

    def OnResize(self, w, h):
        glViewport(0, 0, w, h)

        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()

        gluPerspective(45., \
                       float(w)/float(h), \
                       1., \
                       200.)
        
    
    def update(self, value):
        self._angle += .1
        if self._angle > 360. : self._angle -= 360.
        
        self.pos += 0.08
        while self.pos > self.floorTextureSize :
            self.pos -= self.floorTextureSize
            
        glutPostRedisplay()
        glutTimerFunc(25, self.update, 0)
        

    def handleKeypress(self, key, x, y):
        if ord(key) == 27 : exit()


    def loadTexture(self):
        self.textureId = glGenTextures(1)            
        
        try:
            image = Image.open("checkerboard.bmp")
        except:
            print ("Chargement de la texture impossible : fichier introuvable")
            return
        
        w, h = image.size
        pixels = image.transpose(Image.FLIP_TOP_BOTTOM).tobytes()
        
        glBindTexture(GL_TEXTURE_2D, self.textureId)
        gluBuild2DMipmaps(GL_TEXTURE_2D, \
                          GL_RGB, \
                          w, h, \
                          GL_RGB, \
                          GL_UNSIGNED_BYTE, \
                          pixels)

        del image
        del pixels
        
 
if __name__ == '__main__' :
    canvas = myGLCanvas()

